import { DynamoDBClient } from '@aws-sdk/client-dynamodb';
import { DynamoDBDocumentClient, QueryCommandInput, ScanCommandInput } from '@aws-sdk/lib-dynamodb';
interface SimpleDocumentType {
    id: string;
}
type FindAllByQuery = {
    lastEvaluatedId?: string;
    scanCommandInput: Omit<ScanCommandInput, 'TableName'>;
};
/**
 * A simple repository for DynamoDB
 * @template DocumentType the type of the document
 * example:
 * ```ts
 * type User {
 *  id: string
 *  name: string
 * }
 * class UserRepository extends SimpleDynamoRepository<User> {}
 * const userRepository = new UserRepository(new DynamoDBClient({ region: 'eu-central-1' }), 'users')
 * ```
 */
export declare abstract class SimpleDynamoRepository<DocumentType extends SimpleDocumentType> {
    protected readonly client: DynamoDBDocumentClient;
    protected readonly tableName: string;
    constructor(client: DynamoDBClient, tableName: string);
    /**
     * Find a document by id
     * @param id
     * @throws DocumentNotFoundError if the document does not exist
     * @returns the document
     */
    findOne(id: string): Promise<DocumentType>;
    /**
     * Find a document by the global secondary index
     * @param key the name of the secondary index
     * @param value the value of the secondary index
     * @returns all documents with the given secondary index
     */
    findBy(secondaryKey: string, value: string): Promise<DocumentType[]>;
    /**
     * Find the first document by the global secondary index
     * @param key the name of the secondary index
     * @param value the value of the secondary index
     * @returns the first document with the given secondary index
     */
    findOneBy(secondaryKey: string, value: string): Promise<DocumentType>;
    /**
     * Save a document
     * @param doc
     * @returns the saved document
     */
    save(doc: DocumentType): Promise<DocumentType>;
    /**
     * Delete a document by id
     * @param id
     * @throws DocumentNotFoundError if the document does not exist
     * @returns void
     */
    delete(id: string): Promise<void>;
    /**
     * Find all documents in the table
     * @param limit
     * @param lastEvaluatedId the id of the last document in the previous page
     * @returns all documents in the table
     * example:
     * ```ts
     * let lastEvaluatedId: string | undefined = undefined
     * do {
     *   const users: User[] = await userRepository.findAll(10, lastEvaluatedId)
     *   lastEvaluatedId = users[users.length - 1]?.id || undefined
     * } while (lastEvaluatedId)
     * ```
     */
    findAll(limit?: number, lastEvaluatedId?: string): Promise<DocumentType[]>;
    /**
     * Find all documents in the table by query
     * @param query ScanCommandInput
     * @param lastEvaluatedId the id of the last document in the previous page
     * @returns all documents in the table
     * example:
     * ```ts
     * let lastEvaluatedId: string | undefined = undefined
     * do {
     *   const users: User[] = await userRepository.findAllBy(query, lastEvaluatedId)
     *   lastEvaluatedId = users[users.length - 1]?.id || undefined
     * } while (lastEvaluatedId)
     * ```
     */
    findAllBy(query: FindAllByQuery): Promise<DocumentType[]>;
    /**
     * Count all documents in the table
     * @returns the number of documents in the table
     */
    count(): Promise<number>;
    /**
     * Find by query
     * @param queryCommandInput
     * @returns all documents that match the query
     */
    queryBy(queryCommandInput: Omit<QueryCommandInput, 'TableName'>): Promise<DocumentType[]>;
    /**
     * Find one by query
     * @param queryCommandInput
     * @returns the first document that matches the query
     */
    queryOneBy(queryCommandInput: Omit<QueryCommandInput, 'TableName'>): Promise<DocumentType>;
}
export {};
//# sourceMappingURL=SimpleDynamoRepository.d.ts.map