/// <reference types="node" />
import { S3Client } from '@aws-sdk/client-s3';
interface SimpleFileType {
    key: string;
    content: Uint8Array | string | Buffer;
    contentType: string;
}
/**
 * A simple repository for S3 which is used to save and retrieve files and their metadata
 * @param T the type of the file
 * @param bucketName the name of the bucket
 * @param client the S3 client
 * @example
 * ```ts
 * const s3Client = new S3Client({ region: 'eu-central-1' })
 * const s3Repository = new SimpleS3Repository<SimpleFile>(s3Client, 'my-bucket')
 * ```
 */
export declare class SimpleS3Repository<FileType extends SimpleFileType> {
    private readonly client;
    private readonly bucketName;
    private readonly FIFTEEN_MINUTES;
    constructor(client: S3Client, bucketName: string);
    /**
     * Save a file
     * @param file as SimpleFile
     * @returns void
     * @example
     * ```ts
     * const file: SimpleFile = {
     *  key: 'foo/bar.json',
     *  content: 'my-file-content',
     *  contentType: 'application/json',
     * }
     * await s3Repository.save(file)
     * ```
     */
    save(file: FileType): Promise<void>;
    /**
     * Get a file as string
     * @param filePath the path of the file
     * throws DocumentNotFoundError if the file does not exist
     * @returns the file as string
     */
    getOneAsString(filePath: string): Promise<string>;
    /**
     * Get a file as byte array
     * @param filePath the path of the file
     * throws DocumentNotFoundError if the file does not exist
     * @returns the file as byte array
     */
    getOneAsByteArray(filePath: string): Promise<Uint8Array>;
    /**
     * Get a file as readable stream
     * @param filePath the path of the file
     * throws DocumentNotFoundError if the file does not exist
     * @returns the file as a readable stream
     */
    getOneAsReadableStream(filePath: string): Promise<ReadableStream>;
    private getOne;
    /**
     * Get all which match the prefix
     * @param prefix the prefix
     * @returns all files which match the prefix
     */
    list(prefix: string): Promise<{
        filePath: string;
        lastModified: string;
    }[]>;
    /**
     * Get all metadata for a file
     * @param filePath the path of the file
     * throws DocumentNotFoundError if the file does not exist
     * @returns the metadata of the file
     */
    getOneMetadata(filePath: string): Promise<FileType>;
    /**
     * Delete a file
     * @param filePath the path of the file
     * @returns void
     */
    deleteOne(filepath: string): Promise<void>;
    /**
     * Delete multiple files
     * @param filePaths the paths of the files
     * @returns void
     */
    delete(filePaths: string[]): Promise<void>;
    /**
     * Get a signed download url
     * @param filePath the path of the file
     * @param expiresIn the time in seconds the url is valid
     * @returns the signed download url
     */
    getSignedDownloadUrl(filePath: string, expiresIn?: number): Promise<string>;
    /**
     * Get a signed upload url
     * @param filePath the path of the file
     * @param expiresIn the time in seconds the url is valid
     * @returns the signed upload url
     */
    getSignedUploadUrl(filePath: string, expiresIn?: number): Promise<string>;
}
export {};
//# sourceMappingURL=SimpleS3Repository.d.ts.map